import json
from urllib.parse import urlencode

import fastspider


class TestTmall(fastspider.LightSpider):
	"""
		令牌过期, 需要替换params中data的 pvuuid 值和 sign 以及 headers中的cookies
		获取聚划算分两个接口, params中的data也分两种情况
	"""
	start_urls = [
		"https://h5api.m.tmall.com/h5/mtop.tmall.kangaroo.core.service.route.pagerecommendservice/1.0/?",
	]

	def start_requests(self):
		params = {
			"jsv": "2.6.1",
			"appKey": "12574478",
			"t": "1627898871097",
			"sign": "3e2fd70745b86fa410010deea9c4f937",
			"api": "mtop.tmall.kangaroo.core.service.route.PageRecommendService",
			"v": "1.0",
			"param": """[object Object]""",
			"timeout": "3000",
			"jsonpIncPrefix": "kangaroo",
			"dataType": "jsonp",
			"preventFallback": "true",
			"type": "jsonp",
			"callback": "mtopjsonpkangaroo4",
			# "data": """{"url":"https://pages.tmall.com/wow/ark-pub/common/db6e866b/tpl?spm=a1z10.3-b-s.w15914388-17459263184.11.12c25ce8Iio9Pf&wh_sid=84a0f5c4d1f663e6&sellerId=880734502&scene=taobao_shop","cookie":"hng=CN|zh-CN|CNY|156","pvuuid":"v1-eb25321b-092f-4959-b751-f67ffd7d9e98-1627898864165","fri":"{\\"moduleIdList\\":[\\"3986989690\\",\\"9707198930\\",\\"3305063800\\",\\"7752805340\\",\\"7284623640\\",\\"6491981960\\",\\"8153725620\\",\\"2871549900\\",\\"2094891590\\",\\"6591518700\\",\\"9530138980\\",\\"5254386120\\",\\"8098797430\\",\\"4866466790\\",\\"9494683320\\",\\"5019401070\\",\\"5335728530\\",\\"2432255390\\",\\"3689093040\\"]}","schemaVersion":"c519d06a-55a9-46dc-be04-877ff3f13409","sequence":3,"excludes":"2094891590;2871549900;3305063800;3986989690;5254386120;6491981960;6591518700;7284623640;7752805340;8153725620;9530138980;9707198930","device":"pc","backupParams":"excludes,device"}"""
			"data": """{"url":"https://pages.tmall.com/wow/ark-pub/common/db6e866b/tpl?spm=a1z10.3-b-s.w15914388-17459263184.11.12c25ce8Iio9Pf&wh_sid=84a0f5c4d1f663e6&sellerId=880734502&scene=taobao_shop","cookie":"hng=CN|zh-CN|CNY|156","pvuuid":"v1-eb25321b-092f-4959-b751-f67ffd7d9e98-1627898864165","fri":"{\\"moduleIdList\\":[\\"3986989690\\",\\"9707198930\\",\\"3305063800\\",\\"7752805340\\",\\"7284623640\\",\\"6491981960\\",\\"8153725620\\",\\"2871549900\\",\\"2094891590\\",\\"6591518700\\",\\"9530138980\\",\\"5254386120\\",\\"8098797430\\",\\"4866466790\\",\\"9494683320\\",\\"5019401070\\",\\"5335728530\\",\\"2432255390\\",\\"3689093040\\"]}","schemaVersion":"c519d06a-55a9-46dc-be04-877ff3f13409","sequence":2,"excludes":"3305063800;3986989690;6491981960;7284623640;7752805340;8153725620;9707198930","device":"pc","backupParams":"excludes,device"}"""
		}
		headers = {
			"Host": "h5api.m.tmall.com",
			"Cookie": "cna=z2dgGUY6wVsCAbaUcgOXTPOu; hng=CN%7Czh-CN%7CCNY%7C156; t=dbfb4fe94b6d684f7bc00c3d45fec865; _tb_token_=753851b8f5eee; cookie2=1aaf9e03a3365820865d17902f6f7ef6; xlly_s=1; tk_trace=oTRxOWSBNwn9dPyorMJE%2FoPdY8zfvmw%2Fq5v1XFl6j7SvTEhgOFG8ZODABUcb7AOD%2BiuM61sNTeCFO6cjoyLtZBdyV8Ftg9DjKRTs2MSPx5aXDeVbjtNata3mduWbLVvQd%2FprBYEjxm3FU6MHFjJ1Ek71%2FCmhKMxCYRuRL6FAwVNhLXR8qmIKfWZhoFSzFuiS8a72ZgGcEeKt8SYKhsoI%2Bii3VPn7%2FkvBo6xJbOL8SwQkE5XxSZkoHY%2BViIRcenevY7i9qCdWxOUeAOvl7W6TgFv%2BRCs%3D; miid=248935710892448696; dnk=wang779598160; uc1=cookie15=UIHiLt3xD8xYTw%3D%3D&existShop=false&pas=0&cookie16=Vq8l%2BKCLySLZMFWHxqs8fwqnEw%3D%3D&cookie21=URm48syIYn73&cookie14=Uoe2yt28yU2tzA%3D%3D; uc3=vt3=F8dCujPwtebwX4BQ7ts%3D&nk2=FPjankxl8h7%2FtsBX2w%3D%3D&lg2=UtASsssmOIJ0bQ%3D%3D&id2=VAYrEn2psuPG; tracknick=wang779598160; uc4=id4=0%40Vh%2B56EJYlXZqQsRPzH4qvtG3gis%3D&nk4=0%40FnNat8zj29hCq6XUqDZSFGOK%2FHSEsgXh; lgc=wang779598160; login=true; cancelledSubSites=empty; csg=45796b34; enc=O3Hpc%2Ff36a8PeHONP%2B4YxDgIe0JMVWBfJNDzsbUIfu7jPzlQmUhezqS2wMxbBl6S5c5t6fRjeOlNeR6NvdR99Q%3D%3D; _m_h5_tk=173e91bf1a3e0e97b95b773266ec1b7c_1627903897321; _m_h5_tk_enc=616dc2cfbfd663cf39c45eef0f639447; tfstk=cnbVBQvZ0rUqfxY_vETalGRYkKoAZgCcs4RBmgC5iuCzxFKcibM9ZGukaC9Lrnf..; l=eBOOJLGRjVJholeUBOfZnurza77TIIRAguPzaNbMiOCPOgCM5tx1W6hkwP8HCnGVh6u6R3Rj4pR4BeYBqCmWXFQpwm4ty4Hmn; isg=BOXl1w4T6OOQcQ3lTHinmrcp9KEfIpm0XzHAuefKq5wq_gVwr3BQhVqQiGKIfrFs",
			"sec-ch-ua": 'Not;A Brand";v="99", "Google Chrome";v="91", "Chromium";v="91"',
			"sec-ch-ua-mobile": "?0",
			"user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
			"accept": "*/*",
			"sec-fetch-site": "same-site",
			"sec-fetch-mode": "no-cors",
			"sec-fetch-dest": "script",
			"referer": "https://pages.tmall.com/",
			"accept-language": "zh-CN,zh;q=0.9,zh-TW;q=0.8,en-US;q=0.7,en;q=0.6",
		}

		cookies = {
			'''
				cookie: t=23ba92232b7677dafdee2a748abbfc0f
				cookie: _tb_token_=3b137e1d71e5
				cookie: cookie2=1d6df488e8e33b5cc0894267b95caddc
				cookie: _m_h5_tk=eeb0bb31c0fcc77ee445367c0186231a_1627881573316
				cookie: _m_h5_tk_enc=9eebae74dab94fbd9d168446d10b335e
			'''
		}
		for url in self.start_urls:
			params = urlencode(params)
			new_url = url + params.replace("+", "%20")
			yield fastspider.Request(url=new_url, headers=headers)
			# yield fastspider.Request(url=self.url1, headers=headers)

	def parse(self, request, response):
		text = response.text
		print(text)
		result = json.loads(text[20:-1])

		data = result["data"]["resultValue"]["data"]
		for key, value in data.items():
			print(key)
			if value:
				for goods in value.get("item") or []:
					print(goods["itemTitle"], goods["itemActPrice"])


if __name__ == '__main__':
	t = TestTmall()
	t.start()
# url1 饼干鹏化、果干蜜饯、大口吃肉、坚果炒货、糕点点心 params需要data
# url2 方便速食、糖巧布丁、鱿鱼海味、豆干蔬菜、猜你喜欢 prams需要data
# url3 镇店爆款 params不需要data



# encoding=utf-8
"""
Auth: coco369
Email: 779598160@qq.com

CreateTime: 2021/07/29

Desc: 配置文件
"""

# 爬虫配置
SPIDER_THREAD_COUNT = 1  # 爬虫并发数

# request网络请求超时时间
REQUEST_TIMEOUT = 22  # 等待服务器响应的超时时间，浮点数，或(connect timeout, read timeout)元组

# UserAgent类型 支持 'chrome', 'opera', 'firefox', 'internetexplorer', 'safari'，若不指定则随机类型
USER_AGENT_TYPE = "chrome"

# requests 使用session
USE_SESSION = False

# 下载时间间隔 单位秒。
# 支持格式, 如 SPIDER_SLEEP_TIME = 3
# 如 SPIDER_SLEEP_TIME = [2, 5] 或者 (2, 5) 则间隔为 2~5秒之间的随机数，包含2和5
SPIDER_SLEEP_TIME = (
	0
)

# 是否开启代理
PROXY_ENABLE = True
# 是否使用隧道代理
PROXY_TUNNEL_HOST = None
PROXY_TUNNEL_PORT = None
PROXY_TUNNEL_USER = None
PROXY_TUNNEL_PASSWORD = None

# 浏览器渲染
WEBDRIVER = dict(
	not_load_images=True,  # 是否加载图片
	user_agent=None,  # 字符串 或 无参函数，返回值为user_agent
	proxies=None,  # xxx.xxx.xxx.xxx:xxxx 或 无参函数，返回值为代理地址
	headless=True,  # 是否为无头浏览器
	driver_type="Chrome",  # CHROME、PHANTOMJS、FIREFOX
	timeout=30,  # 请求超时时间
	window_size=(1024, 800),  # 窗口大小
)

# 数据入库的pipeline，可自定义，默认RedisPipeline
ITEM_PIPELINES = [
	# "fastspider.pipeline.test.TmallCheapPipeline"
	# "fastspider.pipeline.redis_pipeline.RedisPipeline",
	# "fastspider.pipeline.mysql_pipeline.MysqlPipeline",
	# "fastspider.pipeline.mongo_pipeline.MongoPipeline",
]

# 导入自定义settings的配置文件
try:
	from settings import *
except Exception as e:
	pass



# encoding=utf-8
"""
Auth: coco369
Email: 779598160@qq.com

CreateTime: 2021/07/29

Desc: fastspider核心代码, 控制器
"""
from threading import Thread


class Controller(Thread):

	def __init__(self):

		pass

	def get_requests(self, count):

		pass



# encoding=utf-8
import json
import re

import fastspider
from fastspider.item.item import Item
from fastspider.utils.tools import cookies2dict


class TmallCheapItem(Item):

	def __init__(self, href="", title="", price="", msg="", src=""):
		self.table_name = "tmall_cheap"
		self.href = href
		self.title = title
		self.price = price
		self.msg = msg
		self.src = src


class TestSpider(fastspider.LightSpider):
	start_urls = [
		"https://pages.tmall.com/wow/ark-pub/common/db6e866b/tpl?spm=a1z10.3-b-s.w20163031-22649597774.17.564348b739A6q4&wh_sid=a9e13bad3df9ada9&sellerId=628189716&scene=taobao_shop"
	]

	detail_api = "https://mdskip.taobao.com/mobile/queryH5Detail.htm?itemId="

	detail_url = "https://detail.tmall.com/item.htm?id="

	def start_requests(self):
		for url in self.start_urls:
			yield fastspider.Request(url=url, callback=self.parse, web_render=True, web_render_scroll=True)

	def parse(self, request, response):
		text = response.content
		patterns = re.compile(r'href="//a.m.taobao.com/i(.*?).htm')
		result = patterns.findall(text)
		cookies = cookies2dict(
			"_tb_token_=e6b5a0e9e3a75;cookie2=199d289165818892c52422ad65a04320;t=d4fea93bd0a64ba2f26f6c098b82f5b6")
		headers = {
			"referer": "https://pages.tmall.com/wow/ark-pub/common/db6e866b/tpl?spm=a1z10.3-b-s.w20163031-22649597774.17.564348b739A6q4&wh_sid=a9e13bad3df9ada9&sellerId=628189716&scene=taobao_shop",
			"user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36",
		}
		for gid in result[:5]:
			tm_detail_api = self.detail_api + gid
			meta = {"gid": gid, "cookies": cookies, "headers": headers}
			yield fastspider.Request(url=tm_detail_api, cookies=cookies, headers=headers, callback=self.parse_detail,
			                         meta=meta, verify=False)

	def parse_detail(self, request, response):
		meta = response.meta
		text = response.content
		data = json.loads(str(text, encoding='gbk').replace('\n', ''))
		if data.get("item"):
			meta["title"] = data["item"]["title"]
			meta["price"] = data["price"]["price"]["priceText"]
			meta["msg"] = data['resource']['bigPromotion']['memo'][0]['text']
			# 获取图片
			tm_detail_url = self.detail_url + meta["gid"]
			meta["href"] = tm_detail_url
			yield fastspider.Request(url=tm_detail_url, cookies=meta["cookies"], headers=meta["headers"],
			                         callback=self.parse_detail_html, meta=meta, verify=False)
		else:
			print("过期")

	def parse_detail_html(self, request, response):
		meta = response.meta
		src = response.xpath('//*[@id="J_UlThumb"]/li[1]/a/img/@src').get()
		print('-----------------', src, meta["title"], meta["price"], meta["msg"], meta["href"])
		item = TmallCheapItem(src=src, title=meta["title"], price=meta["price"], msg=meta["msg"], href=meta["href"])
		yield item


if __name__ == "__main__":
	TestSpider(5).start()



# encoding=utf-8
"""
Auth: coco369
Email: 779598160@qq.com

CreateTime: 2021/07/29

Desc: fastspider核心爬虫AirSpider的基类代码
"""
from fastspider.http.request.request import Request


class LightBase(object):

	def start_requests(self):
		"""
			解析url地址
		:return: 返回可迭代的yield Request()
		"""
		if hasattr(self, "start_urls") and not self.start_urls:
			raise Exception("Spider 必须定义 start_urls")
		for url in self.start_urls:
			yield Request(url=url)

	def parser(self, request, response):
		"""
			默认的解析函数, 解析响应response的内容
		:param request: 请求对象
		:param response: 响应对象
		:return: 可不返回内容, 或者返回Item对象
		"""
		pass

	def start_callback(self):
		pass

	def end_callback(self):

		pass

	@property
	def name(self):
		# 示例方法, 获取类名
		return self.__class__.__name__



# encoding=utf-8

import fastspider
from fastspider.item.item import Item


class DoubanItem(Item):

	def __init__(self, href, title):
		self.table_name = "douban"
		self.href = href
		self.title = title


class TestSpider(fastspider.LightSpider):
	start_urls = ["https://movie.douban.com/top250"]

	def parser(self, request, response):
		print(response)
		movies = response.xpath('//*[@id="content"]/div/div[1]/ol/li')
		for movie in movies:
			href = movie.xpath('./div/div[2]/div[1]/a/@href')[0].get()
			title = movie.xpath('./div/div[2]/div[1]/a/span[1]/text()')[0].get()

			print(href, title)
			# douban_item = DoubanItem(href, title)
			# yield douban_item


if __name__ == "__main__":
	TestSpider(3).start()



# encoding=utf-8
"""
Auth: coco369
Email: 779598160@qq.com

CreateTime: 2021/07/29

Desc: fastspider核心代码, 控制器
"""
import time
from collections import Iterable
import random
from threading import Thread

from fastspider.http.request.request import Request
from fastspider.item.item import Item
from fastspider.settings import common


class BaseController(Thread):
	# 统计任务执行成功的个数 和 任务执行失败的个数, 用于后期统计任务执行成功率, 成功率太低, 则发送报警信息
	# 成功率计算公司  失败任务总数 / 总任务数 > 0.5 则发送报警信息
	_success_task_num = 0
	_fail_task_num = 0

	has_task_flag = False

	def __init__(self):
		super(BaseController, self).__init__()

		self._thread_stop = False
		self._parser = []

	def has_task(self):
		return self.has_task_flag

	def stop(self):
		# 将线程启动的标识 设置为True
		self._thread_stop = True

	def add_parser(self, parser):
		self._parser.append(parser)


class AirSpiderController(BaseController):

	def __init__(self, memory_db, item_cache):
		super(AirSpiderController, self).__init__()

		self._memory_db = memory_db
		self._item_cache = item_cache

	def run(self):

		while not self._thread_stop:
			request = self._memory_db.get_nowait()
			if not request:
				self.has_task_flag = False
			else:
				self.has_task_flag = True
				self.deal_requests([request])

	def deal_requests(self, requests):
		"""
			处理请求
		:param request: 请求
		:return:
		"""
		response = None
		for request in requests:
			for parser in self._parser:
				# 判断当前请求是否是请求对应的爬虫触发的。避免同时多个爬虫一起执行时, 请求request和对应的爬虫匹配错误
				if request.parser_name == parser.name:
					try:
						# 解析request
						# if request.download_middleware:
						# 	pass
						# else:
						response = request.get_response()
						# 判断是否有回调函数
						if request.callback:
							# 检查回调函数是否可用
							# callback_parser = (
							# 	request.callback if callable(request.callback)
							# 	else tools.check_class_method(parser, request.callback)
							# )
							callback_parser = (request.callback)
							results = callback_parser(request, response)
						else:
							results = parser.parser(request, response)

						if results and not isinstance(results, Iterable):
							raise Exception(f"{parser.name}.{request.callback}必须可迭代的返回值")

						for result in results or []:
							if isinstance(result, Request):
								result.parser_name = result.parser_name or parser.name
								# 如果是同步的callback, 将解析request对象添加到requests中
								if result.request_sync:
									requests.append(result)
								else:
									# 异步, 将任务添加到 任务队列中
									self._memory_db.put(result)
							elif isinstance(result, Item):
								# 存入item缓存中
								self._item_cache.put(result)

					except Exception as e:
						# TODO: 记录任务失败的信息
						print(e)
						self._memory_db.put(request)
						# if request.retry_time:
						# 	request.retry_time -= 1
						# 	self._memory_db.put(request)
					# finally:
					# 	print("释放相关的链接, 如数据库、浏览器的链接")

					break
		# 休眠
		if common.SPIDER_SLEEP_TIME:
			if isinstance(common.SPIDER_SLEEP_TIME, (tuple, list)) and len(common.SPIDER_SLEEP_TIME) == 2:
				sleep_times = random.randint(common.SPIDER_SLEEP_TIME[0], common.SPIDER_SLEEP_TIME[1])
				time.sleep(sleep_times)
			else:
				time.sleep(common.SPIDER_SLEEP_TIME)



# encoding=utf-8
import json
import re
import time

import fastspider
from fastspider.utils.tools import cookies2dict


class TestSpider(fastspider.LightSpider):
	start_urls = [
		"https://detail.tmall.com/item.htm?id=13060317764"
	]

	def start_requests(self):
		headers = {
			"referer": "https://pages.tmall.com/wow/ark-pub/common/db6e866b/tpl?spm=a1z10.3-b-s.w20163031-22649597774.17.564348b739A6q4&wh_sid=a9e13bad3df9ada9&sellerId=628189716&scene=taobao_shop",
			"user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36",
		}
		for url in self.start_urls:
			yield fastspider.Request(url=url, callback=self.parse, headers=headers)

	def parse(self, request, response):
		print(response)

		src = response.xpath('//*[@id="J_UlThumb"]/li[1]/a/img/@src').get()
		print(src)


if __name__ == "__main__":
	TestSpider().start()



# encoding=utf-8
import json
import re
import time

import requests

import fastspider
from fastspider.utils.tools import cookies2dict


class TestSpider(fastspider.LightSpider):
	start_urls = [
		# "https://mdskip.taobao.com/core/initItemDetail.htm?itemId=13060317764"
		"https://mdskip.taobao.com/mobile/queryH5Detail.htm?itemId=13060317764"
	]

	def start_requests(self):
		# cookies = cookies2dict("enc=He0FFUfznnffpjS6YSXQiTktURoDYk0on8ZMuumJrVPIKvBtMFgThw7BQ%2F97zg%2BQ7aFqB86eSU%2F5SNj8c%2FThpQ%3D%3D;x5sec=7b226d616c6c64657461696c736b69703b32223a226330303336333135643931663031363236386133656634396666623265323538434c696670496747454f794f6c72502b362b6d397577456f416a443874736d4e41673d3d227d")
		cookies = cookies2dict("_tb_token_=e6b5a0e9e3a75; cookie2=199d289165818892c52422ad65a04320; t=d4fea93bd0a64ba2f26f6c098b82f5b6")
		headers = {
			"referer": "https://detail.tmall.com/",
			"user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36",
			"Accept": "*/*",
			"Accept-Encoding": "gzip, deflate, br",
			"Connection": "keep-alive"
		}
		for url in self.start_urls:
			# print('111111')
			# proxies = requests.get("http://139.196.28.232:8090/proxy?key=proxy").json()
			# print(proxies)
			meta = {
				"cookies": cookies,
				"headers": headers
			}
			yield fastspider.Request(url=url, callback=self.parse, headers=headers, meta=meta)
			# yield fastspider.Request(url=url, callback=self.parse, cookies=cookies, headers=headers, meta=meta)

	def parse(self, request, response):
		meta = response.meta
		print(meta)
		text = response.content
		data = json.loads(str(text, encoding='gbk').replace('\n', ''))
		print(data)
		if data.get("defaultModel"):
			print(data["defaultModel"]["detailPageTipsDO"]["warmingUpHintText"])
		else:
			print("过期")


if __name__ == "__main__":
	TestSpider().start()



# encoding=utf-8
"""
Auth: coco369
Email: 779598160@qq.com

CreateTime: 2021/07/29

Desc: fastspider核心代码, 轻量级爬虫air_spider
"""
from threading import Thread

from fastspider.cache.item_cache import ItemCache
from fastspider.core.base.light_base import LightBase
from fastspider.core.controller.spider_controller import AirSpiderController
from fastspider.db.memory_db import MemoryDB
from fastspider.http.request.request import Request
from fastspider.settings import common


class LightSpider(LightBase, Thread):
	# 定义私有变量
	__common_settings__ = {}

	def __init__(self, thread_count=None):
		"""
			初始化配置
			用户可自定义settings配置, 配置变量为__common_settings__, 类型为dict
		"""
		super(LightSpider, self).__init__()

		self._memory_db = MemoryDB()
		self._item_cache = ItemCache(redis_key="light_spider")

		self._parser_controller = []

		for k, v in self.__class__.__common_settings__.items():
			setattr(common, k, v)
		self._thread_count = common.SPIDER_THREAD_COUNT if not thread_count else thread_count

	def add_task(self):
		"""
			将start_response中的任务Request对象存储到内存队列中
		:return: None
		"""
		for req in self.start_requests():
			if not isinstance(req, Request):
				raise Exception("返回参数错误, 当前只支持yield fastspider.Request对象")

			# 将当前请求的类名添加到request对象上
			req.parser_name = req.parser_name or self.name
			self._memory_db.put(req)

	def all_thread_task_done(self):
		"""
			判断内存队列中的任务是否执行完毕
		:return: True: 已执行完所有任务。 False: 还有处于待执行中的任务
		"""
		for i in range(3):
			for parser_controller in self._parser_controller:
				if parser_controller.has_task():
					return False

			# 检测request任务队列是否为空
			if not self._memory_db.is_empty():
				return False

			# 检查item任务队列是否为空
			if not self._item_cache.is_empty() or self._item_cache.adding_item_to_db():
				return False

		return True

	def run(self):
		"""
			启动线程
		"""
		# 先将任务监听的控制方法启动, 再通过add_task将需要爬取的request对象写入, 通过while检测任务执行的情况, 任务执行完, 则暂定任务_thread_stop设置为True
		for i in range(self._thread_count):
			spider_controller = AirSpiderController(self._memory_db, self._item_cache)
			# 将当前执行的爬虫实例对象添加到线程中
			spider_controller.add_parser(self)
			spider_controller.start()

			self._parser_controller.append(spider_controller)

		self._item_cache.start()

		# 将任务放在内存中
		self.add_task()

		# 死循环, 一直执行任务, 判断任务task是否执行完成, 如果任务执行完毕, 则关闭各种链接，如mysql, 浏览器对象
		while True:
			if self.all_thread_task_done():

				# 暂停解析爬虫任务线程
				for parser_controller in self._parser_controller:
					parser_controller.stop()

				# 暂停解析item任务线程
				self._item_cache.stop()

				print("无任务, 爬虫执行完毕")
				break
